@@ -5,9 +5,11 @@ use warnings;
 use Mojo::Base 'Mojolicious::Plugin';
 use Mojo::JSON qw(decode_json to_json);
 use GraphQL::Execution qw(execute);
+use GraphQL::Type::Library -all;
 use Module::Runtime qw(require_module);
+use Mojo::Promise;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 my @DEFAULT_METHODS = qw(get post);
 my $EXECUTE = sub {
@@ -20,6 +22,21 @@ my $EXECUTE = sub {
     $variables,
     $operationName,
     $field_resolver,
+    # promise code - not overridable
+    {
+      all => sub {
+        # current Mojo::Promise->all only works on promises, force that
+        my @promises = map is_Promise($_)
+          ? $_ : Mojo::Promise->new->resolve($_),
+          @_;
+        # only actually works when first promise-instance is a
+        # Mojo::Promise, so force it to be one. hoping will be fixed soon
+        Mojo::Promise->new->resolve->all(@promises)->then(sub { shift; @_ })
+      },
+      # currently only instance methods. not wasteful at all.
+      resolve => sub { Mojo::Promise->new->resolve(@_) },
+      reject => sub { Mojo::Promise->new->reject(@_) },
+    },
   );
 };
 sub make_code_closure {
@@ -81,6 +98,7 @@ sub register {
     my $body = decode_json($c->req->body);
     my $data = eval { $handler->($c, $body, $EXECUTE) };
     $data = { errors => [ { message => $@ } ] } if $@;
+    return $data->then(sub { $c->render(json => shift) }) if is_Promise($data);
     $c->render(json => $data);
   };
   push @{$app->renderer->classes}, __PACKAGE__
@@ -154,6 +172,12 @@ Mojolicious::Plugin::GraphQL - a plugin for adding GraphQL route handlers
 This plugin allows you to easily define a route handler implementing a
 GraphQL endpoint.
 
+As of version 0.09, it will supply the necessary C<promise_code>
+parameter to L<GraphQL::Execution/execute>. This means your resolvers
+can (and indeed should) return Promise objects to function
+asynchronously. Notice not necessarily "Promises/A+" - all that's needed
+is a two-arg C<then> to work fine with GraphQL.
+
 The route handler code will be compiled to behave like the following:
 
 =over 4
@@ -161,7 +185,9 @@ The route handler code will be compiled to behave like the following:
 =item *
 
 Passes to the L<GraphQL> execute, possibly via your supplied handler,
-the given schema, C<$root_value> and C<$field_resolver>.
+the given schema, C<$root_value> and C<$field_resolver>. Note as above
+that the wrapper used in this plugin will supply the hash-ref matching
+L<GraphQL::Type::Library/PromiseCode>.
 
 =item *
 
